00001  IDENTIFICATION DIVISION.
00002  PROGRAM-ID.    GNUDBLD.
00003 *AUTHOR.        CARL D. GOLDIN.
00004 *DATE-WRITTEN.  Jan 05, 2020.
00005 *LAST-REVISION. Jan 08, 2020.
00006
00007 * Copyright (c) Carl D. Goldin, 2020. <cdgoldin@gmail.com>
00008
00009 * This program is free software; you can redistribute it and/or
00010 * modify it under the terms of the GNU General Public License
00011 * as published by the Free Software Foundation; either
00012 * version 2, or (at your option) any later version.
00013 *
00014 * This program is distributed in the hope that it will be
00015 * useful, but WITHOUT ANY WARRANTY; without even the implied
00016 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
00017 * PURPOSE.  See the GNU General Public License for more
00018 * details.
00019 *
00020 * You should have received a copy of the GNU General Public
00021 * License along with this software; see the file COPYING.
00022 * If not, write to the Free Software Foundation, Inc.,
00023 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
00024
00025 ******************************************************************
00026 * GNUDBLD - Load ISAM file from sequential or null file          *
00027 *           Append to an existing ISAM file from sequntial file  *
00028 *        or Dump ISAM files to sequential file                   *
00029 ******************************************************************
00030 * Syntax:                                                        *
00031 * -------                                                        *
00032 * gnudbld load   seq-path isam-path select fd record output_path *
00033 *         append seq-path isam-path select fd record output_path *
00034 *         dump   isam-path seq-path select fd record output_path *
00035 *                                                                *
00036 * load/append/dump specify the action to be taken. Append is     *
00037 *    is like load, except that records from the sequential file  *
00038 *    are appended to an existing ISAM file. This can be useful   *
00039 *    for merging data from various sources.                      *
00040 *                                                                *
00041 * seq-path is the pathname of the sequential file from which the *
00042 *    ISAM file is to be loaded, or to which the ISAM file is to  *
00043 *    be dumped.                                                  *
00044 *                                                                *
00045 * isam-path is the pathname of the ISAM file which is to be      *
00046 *    be loaded from or dumped to the sequential file.            *
00047 *                                                                *
00048 * select is the pathname of the copylib file containing the      *
00049 *    SELECT..ASSIGN statement for the ISAM file.                 *
00050 *                                                                *
00051 * fd is the pathname of the copylib file containing the file     *
00052 *    definition statement for the ISAM file. It is not necessary *
00053 *    to provide an FD, unless it has essential information, but  *
00054 *    you must specify "NOFD" if it is omitted, since all the     *
00055 *    parameters are positional.                                  *
00056 *                                                                *
00057 * record is the pathname of the copylib file containing the      *
00058 *    record definition for the ISAM file. The record definition  *
00059 *    MUST contain all the keys specified in the SELECT copylib   *
00060 *    file, and must define the maximum record size in the case   *
00061 *    of variable length files with multiple record definitions.  *
00062 *    Unless the record length is contained in the record, the    *
00063 *    program will dump and load all the records with the maximum *
00064 *    record size, because it has no way of determining the       *
00065 *    actual record size without additional information.          *
00066 *                                                                *
00067 * outpath-path is the pathname where the program will save the   *
00068 *    Cobol load/dump program it generates, INSTEAD of running it.*
00069 *    This allows the user to add specific conversion or variable *
00070 *    record code to the program before running it. If omitted,   *
00071 *    the program will create a file named "gnudbldx.cob", and    *
00072 *    execute it.                                                 *
00073 *                                                                *
00074 *    The sequential and ISAM path names are passed to the        *
00075 *    generated Cobol program via two environment variables:      *
00076 *    SEQ_PATH_NAME and ISAM_PATH_NAME. This is done so that the  *
00077 *    generated Cobol program can be fixed-format in order to be  *
00078 *    compatible with earlier GnuCOBOL versions. However, the     *
00079 *    sequence number and program-id fields are left blank, and   *
00080 *    an asterisk in column 7 (denoting a comment) is changed to  *
00081 *    "*>" so that the file is effectively a free-format file as  *
00082 *    well. And, the text of (free-format) copylib members longer *
00083 *    than 65 characters will still be included in the generated  *
00084 *    file (which can then only be processed as "free-format")    *
00085 *                                                                *
00086 * All pathnames follow the Windoze/Linux convention, i.e.        *
00087 *    1) filename is a file in the current directory.             *
00088 *    2) folder\filename (folder/filename) is a file within the   *
00089 *       specified folder                                         *
00090 *    3) d:\folder\filename (/device/folder/filenameis a fully-   *
00091 *       qualified path name.                                     *
00092 *    4) etc.                                                     *
00093 *                                                                *
00094 * All "pathname" parameters may alternately specify the name of  *
00095 *    an environment variable in which case the program will use  *
00096 *    the value of the environment variable as the pathname.      *
00097 *                                                                *
00098 * In Linux systems, the pathnames and environment variables are  *
00099 *    case sensitive. With Windoze, they are NOT.                 *
00100 *                                                                *
00101 * Copylib members are selected according to the following order  *
00102 *    ie. we use the first file that matches the criteria:        *
00103 *       1) If the pathname exists, use that file.                *
00104 *       2) If the file exists in the current folder, use it.     *
00105 *       3) If the file exists in the COBCPY folder, use it.      *
00106 *       4) If the file exists in the COB_COPY_DIR, use it.       *
00107 *                                                                *
00108 *    Within the current directory, COBCPY and COB_COPY_DIR       *
00109 *    folders the order is: filename, filename.cob, filename.cbl, *
00110 *    filename.cpy                                                *
00111 *                                                                *
00112 * IT IS ASSUMED THAT all COPYLIB FILES ARE line-sequential and   *
00113 * no more than 120-CHARACTERS wide! They may be fixed or free-   *
00114 * format.                                                        *
00115 *                                                                *
00116 * If columns 1-5 of the first record in a copylib file are       *
00117 * numeric, it is assumed that the file is fixed-format (and only *
00118 * columns 7-72 are used). Otherwise, it is assumed that the file *
00119 * is free-format.                                                *
00120 *                                                                *
00121 * Although the program does SOME error-checking, there is an     *
00122 * assumption that your copylib members contain valid code.       *
00123 *                                                                *
00124 * Similarly, this program does NOT check that the sequential and *
00125 * ISAM path names are valid, but the generated program does.     *
00126 *                                                                *
00127 * PARAMETERS ARE POSITIONAL AND MUST BE ENTERED IN THE CORRECT   *
00128 * ORDER AS SHOWN ABOVE!!!                                        *
00129 *                                                                *
00130 * Return Codes:                                                  *
00131 *   00 = normal completion                                       *
00132 *   04 = warning                                                 *
00133 *   08 = severe error (execution terminated)                     *
00134 ******************************************************************
00135  ENVIRONMENT DIVISION.
00136  CONFIGURATION SECTION.
00137  SOURCE-COMPUTER. IBM-PC.
00138  OBJECT-COMPUTER. IBM-PC.
00139
00140  INPUT-OUTPUT SECTION.
00141  FILE-CONTROL.
00142      SELECT COPYLIB-FILE ASSIGN TO COPYLIB-PATH-NAME
00143             ORGANIZATION IS LINE SEQUENTIAL
00144             FILE STATUS IS COPYLIB-FILE-STATUS.
00145
00146      SELECT COBOL-FILE ASSIGN TO COBOL-PATH-NAME
00147             ORGANIZATION IS LINE SEQUENTIAL.
00148
00149  DATA DIVISION.
00150
00151  FILE SECTION.
00152
00153  FD  COPYLIB-FILE.
00154
00155  01  COPYLIB-RECORD.
00156      02  SEQUENCE-NUMBER         PIC 9(05).
00157      02  COLUMN-1-3 REDEFINES SEQUENCE-NUMBER PIC X(3).
00158      02  COMMENT-TEXT.
00159          03  COLUMN-6            PIC X(01).
00160          03  COLUMN-7            PIC X(01).
00161          03  COPYLIB-TEXT        PIC X(65).
00162      02  COPYLIB-ID              PIC X(08).
00163      02  FILLER                  PIC X(40). *> see comment below
00164
00165  FD  COBOL-FILE.
00166
00167  01  COBOL-RECORD.
00168      02  FILLER                  PIC X(07).
00169      02  FREE-FORMAT-TEXT.
00170          03  COBOL-TEXT          PIC X(65).
00171          03  COLUMNS-73-PLUS     PIC X(48). *> see comment below
00172
00173 * the record length is larger than the 80-character fixed-format, so
00174 * that copylib members > 65 characters can be placed in the file, in
00175 * which case it becomes a "free-format file"
00176
00177  WORKING-STORAGE SECTION.
00178
00179  01  COPYRIGHT-NOTICE-1.
00180      02  FILLER          PIC X(20) VALUE "GNUDBLD Release 1.0".
00181  01  COPYRIGHT-NOTICE-2.
00182      02  FILLER          PIC X(09) VALUE SPACES.
00183      02  FILLER          PIC X(14) VALUE "Copyright (c) ".
00184      02  HOLDER          PIC X(22) VALUE "Carl D. Goldin, 2020. ".
00185      02  RIGHTS          PIC X(20) VALUE " ".
00186
00187 ******************************************************************
00188 * Parameters obtained from the command line                      *
00189 ******************************************************************
00190  01  LOAD-OR-DUMP            PIC X(08)   VALUE SPACES.
00191  01  SEQ-PATH-NAME           PIC X(80)   VALUE SPACES.
00192  01  ISAM-PATH-NAME          PIC X(80)   VALUE SPACES.
00193  01  SWAP-PATH-NAME          PIC X(80)   VALUE SPACES.
00194  01  SELECT-PATH-NAME        PIC X(80)   VALUE SPACES.
00195  01  FD-PATH-NAME            PIC X(80)   VALUE SPACES.
00196  01  RECORD-PATH-NAME        PIC X(80)   VALUE SPACES.
00197  01  OUTPUT-PATH-NAME        PIC X(80)   VALUE SPACES.
00198
00199 ******************************************************************
00200 * Parameters used to build the Cobol Program                     *
00201 ******************************************************************
00202  01  SELECT-ARRAY1.
00203      02  SELECT-ARRAY1-LINE PIC X(120) OCCURS 10  INDEXED BY S1X.
00204  01  FILLER REDEFINES SELECT-ARRAY1.
00205      02  SELECT-ARRAY1-CHAR PIC X(01) OCCURS 1200 INDEXED BY S1Y.
00206
00207  01  SELECT-ARRAY-LINES      PIC S9(4) COMP VALUE 10.
00208  01  SELECT-ARRAY-CHARACTERS PIC S9(4) COMP VALUE 1200.
00209
00210  01  SELECT-ARRAY2.
00211      02  SELECT-ARRAY2-LINE PIC X(120) OCCURS 10  INDEXED BY S2X.
00212  01  FILLER REDEFINES SELECT-ARRAY2.
00213      02  SELECT-ARRAY2-CHAR PIC X(01) OCCURS 1200 INDEXED BY S2Y.
00214
00215  01  FD-ARRAY.
00216      02  FD-LINE             PIC X(120) OCCURS 10  INDEXED BY FDX.
00217  01  FILLER REDEFINES FD-ARRAY.
00218      02  FD-CHAR             PIC X(01) OCCURS 1200 INDEXED BY FDY.
00219
00220  01  FD-ARRAY-LINES          PIC S9(4) COMP VALUE 10.
00221  01  FD-ARRAY-CHARACTERS     PIC S9(4) COMP VALUE 1200.
00222
00223  01  COPYLIB-PATH-NAME       PIC X(80) VALUE SPACES.
00224  01  COPYLIB-MEMBER          PIC X(80) VALUE SPACES.
00225  01  COPYLIB-FILE-STATUS     PIC 99    VALUE ZERO.
00226
00227  01  COBCPY-FOLDER           PIC X(80) VALUE SPACES.
00228  01  COB-COPY-DIR-FOLDER     PIC X(80) VALUE SPACES.
00229
00230  01  COBOL-PATH-NAME         PIC X(80) VALUE "GNUDBLDX.COB".
00231
00232  01  OS                      PIC X(20)   VALUE SPACES.
00233
00234  01  REPLY                   PIC X       VALUE SPACES.
00235  01  EOF-SW                  PIC X       VALUE ZERO.
00236
00237  01  I                       PIC S9(4)   COMP VALUE ZERO.
00238  01  J                       PIC S9(4)   COMP VALUE ZERO.
00239  01  ICOUNT                  PIC S9(4)   COMP VALUE ZERO.
00240  01  ALT-KEY-COUNT           PIC S9(4)   COMP VALUE ZERO.
00241  01  CHARACTER-LIMIT         PIC S9(4)   COMP VALUE ZERO.
00242
00243  01  IS-SWITCH               PIC X       VALUE SPACES.
00244  01  TWO-SPACES              PIC XX      VALUE SPACES.
00245  01  SUPPRESS-01-REC         PIC X       VALUE SPACES.
00246  01  SUPPRESS-FD-REC         PIC X       VALUE SPACES.
00247  01  FIXED-FORMAT            PIC X       VALUE SPACES.
00248  01  CREATE-FREE-FORMAT-FILE PIC X       VALUE SPACES.
00249
002506******************************************************************
00251 * Pointers and based variables                                   *
00252 ******************************************************************
00253  01  PTR1                     USAGE IS POINTER.
00254  01  PTR2                     USAGE IS POINTER.
00255
00256  01  TEST-FIELD PIC X(120)  BASED.
00257  01  TEST-FIELD-03 REDEFINES TEST-FIELD PIC X(03).
00258  01  TEST-FIELD-04 REDEFINES TEST-FIELD PIC X(04).
00259  01  TEST-FIELD-07 REDEFINES TEST-FIELD PIC X(07).
00260  01  TEST-FIELD-11 REDEFINES TEST-FIELD PIC X(11).
00261  01  TEST-FIELD-14 REDEFINES TEST-FIELD PIC X(14).
00262  01  TEST-FIELD-16 REDEFINES TEST-FIELD PIC X(16).
00263  01  TEST-FIELD-17 REDEFINES TEST-FIELD PIC X(17).
00264  01  TEST-FIELD-21 REDEFINES TEST-FIELD PIC X(21).
00265  01  TEST-FIELD-26 REDEFINES TEST-FIELD PIC X(26).
00266
00267 ******************************************************************
00268 * GNUDBLD Mainline Process                                       *
00269 ******************************************************************
00270  PROCEDURE DIVISION.
00271
00272  000-MAINLINE SECTION.
00273  000-ENTRY.
00274      DISPLAY " ".
00275      DISPLAY COPYRIGHT-NOTICE-1.
00276      DISPLAY COPYRIGHT-NOTICE-2.
00277      DISPLAY " ".
00278
00279      MOVE ZERO                   TO  RETURN-CODE.
00280
00281 *    Determine if we are running under Windoze or Linux
00282
00283      ACCEPT OS FROM ENVIRONMENT "OS".
00284
00285      IF  OS = SPACES MOVE "LINUX" TO OS, END-IF.
00286
00287 *    Get parameters from command line
00288
00289      ACCEPT LOAD-OR-DUMP       FROM  ARGUMENT-VALUE
00290      ACCEPT  SEQ-PATH-NAME     FROM  ARGUMENT-VALUE
00291      ACCEPT ISAM-PATH-NAME     FROM  ARGUMENT-VALUE
00292      ACCEPT SELECT-PATH-NAME   FROM  ARGUMENT-VALUE
00293      ACCEPT FD-PATH-NAME       FROM  ARGUMENT-VALUE
00294      ACCEPT RECORD-PATH-NAME   FROM  ARGUMENT-VALUE
00295      ACCEPT OUTPUT-PATH-NAME   FROM  ARGUMENT-VALUE
00296
00297      IF  LOAD-OR-DUMP = "DUMP"  *> SWAP THE FILE NAMES!
00298          MOVE  SEQ-PATH-NAME     TO  SWAP-PATH-NAME
00299          MOVE ISAM-PATH-NAME     TO   SEQ-PATH-NAME
00300          MOVE SWAP-PATH-NAME     TO  ISAM-PATH-NAME
00301      END-IF.
00302
00303 *    Get parameters from environment if present
00304
00305      ACCEPT SWAP-PATH-NAME FROM ENVIRONMENT SEQ-PATH-NAME.
00306
00307      IF  SWAP-PATH-NAME NOT = SPACES
00308          MOVE SWAP-PATH-NAME TO SEQ-PATH-NAME
00309      END-IF.
00310
00311      ACCEPT SWAP-PATH-NAME FROM ENVIRONMENT ISAM-PATH-NAME.
00312
00313      IF  SWAP-PATH-NAME NOT = SPACES
00314          MOVE SWAP-PATH-NAME TO ISAM-PATH-NAME
00315      END-IF.
00316
00317      ACCEPT SWAP-PATH-NAME FROM ENVIRONMENT SELECT-PATH-NAME.
00318
00319      IF  SWAP-PATH-NAME NOT = SPACES
00320          MOVE SWAP-PATH-NAME TO SELECT-PATH-NAME
00321      END-IF.
00322
00323      ACCEPT SWAP-PATH-NAME FROM ENVIRONMENT FD-PATH-NAME.
00324
00325      IF  SWAP-PATH-NAME NOT = SPACES
00326          MOVE SWAP-PATH-NAME TO FD-PATH-NAME
00327      END-IF.
00328
00329      ACCEPT SWAP-PATH-NAME FROM ENVIRONMENT RECORD-PATH-NAME.
00330
00331      IF  SWAP-PATH-NAME NOT = SPACES
00332          MOVE SWAP-PATH-NAME TO RECORD-PATH-NAME
00333      END-IF.
00334
00335      ACCEPT SWAP-PATH-NAME FROM ENVIRONMENT OUTPUT-PATH-NAME.
00336
00337      IF  SWAP-PATH-NAME NOT = SPACES
00338          MOVE SWAP-PATH-NAME TO OUTPUT-PATH-NAME
00339      END-IF.
00340
00341 *    Verify parameters and begin to process them
00342
00343      MOVE FUNCTION UPPER-CASE (LOAD-OR-DUMP) TO LOAD-OR-DUMP
00344
00345      IF  NOT (LOAD-OR-DUMP = "LOAD" OR "DUMP" OR "APPEND")
00346          DISPLAY " "
00347          DISPLAY
00348       "Error: first parameter MUST be 'LOAD' 'DUMP' OR 'APPEND'"
00349          MOVE 08 TO RETURN-CODE
00350          DISPLAY " "
00351          STOP RUN.
00352
00353      *> We will let the generated program verify that SEQ and ISAM
00354      *> path names are valid, and that the input file exists, and
00355      *> output file doesn't exist (or that it is OK to overwrite)
00356
00357      PERFORM 100-COPYLIB-PROCESS
00358
00359      PERFORM 200-GENERATE-COBOL-PROGRAM.
00360
00361      DISPLAY " "
00362      DISPLAY "GNUDBLD - End of job"
00363      DISPLAY " ".
00364
00365      STOP RUN.
00366
00367  100-COPYLIB-PROCESS SECTION.
00368  100-ENTRY.
00369      ACCEPT COBCPY-FOLDER       FROM ENVIRONMENT "COBCPY".
00370      ACCEPT COB-COPY-DIR-FOLDER FROM ENVIRONMENT "COB_COPY_DIR".
00371
00372      PERFORM 110-SELECT-PROCESS.
00373
00374      PERFORM 120-FD-PROCESS.
00375
00376      PERFORM 130-RECORD-PROCESS.
00377
00378  100-EXIT.
00379      EXIT.
00380
00381 ******************************************************************
00382 * Process SELECT parameter and copylib file
00383 ******************************************************************
00384  110-SELECT-PROCESS SECTION.
00385  110-ENTRY.
00386      MOVE SELECT-PATH-NAME TO COPYLIB-MEMBER.
00387
00388      PERFORM 140-OPEN-COPYLIB-FILE.
00389
00390      MOVE SPACES                 TO  SELECT-ARRAY1
00391
00392      PERFORM WITH TEST AFTER VARYING S1X FROM 1 BY 1
00393                UNTIL S1X > SELECT-ARRAY-LINES
00394                   OR COPYLIB-RECORD = HIGH-VALUES
00395          PERFORM 220-READ-COPYLIB-RECORD
00396          IF  COPYLIB-RECORD NOT = HIGH-VALUES
00397              MOVE COPYLIB-RECORD     TO  SELECT-ARRAY1-LINE (S1X)
00398          END-IF
00399      END-PERFORM.
00400
00401      CLOSE COPYLIB-FILE
00402
00403      IF  COPYLIB-RECORD NOT = HIGH-VALUES
00404          DISPLAY " "
00405          DISPLAY "Error: More than " SELECT-ARRAY-LINES
00406                  " lines in SELECT copylib statement!"
00407          MOVE 08 TO RETURN-CODE
00408          DISPLAY " "
00409          STOP RUN
00410      END-IF.
00411
00412      MOVE FUNCTION UPPER-CASE (SELECT-ARRAY1) TO SELECT-ARRAY1.
00413      INSPECT SELECT-ARRAY1 REPLACING ALL "." BY " "
00414      INSPECT SELECT-ARRAY1 REPLACING ALL "," BY " "
00415
00416      MOVE ZERO                   TO  ICOUNT.
00417      INSPECT SELECT-ARRAY1 TALLYING  ICOUNT
00418              FOR CHARACTERS BEFORE " INDEXED ".
00419
00420      IF  ICOUNT = SELECT-ARRAY-CHARACTERS
00421          DISPLAY " "
00422          DISPLAY
00423          "Error: SELECT statement is not for indexed file!"
00424          MOVE 08 TO RETURN-CODE
00425          DISPLAY " "
00426          STOP RUN
00427      END-IF.
00428
00429 *    Retain only the listed SELECT entries, as the others will
00430 *    be generated by the program (and won't necessarily match
00431 *    the copylib SELECT statement
00432
00433      SET PTR1                    TO  ADDRESS OF SELECT-ARRAY1
00434      SET S1Y, S2X                TO  1
00435      PERFORM UNTIL S1Y > SELECT-ARRAY-CHARACTERS
00436          SET ADDRESS OF TEST-FIELD TO PTR1
00437
00438          IF  (TEST-FIELD-11 = "RECORD KEY ")
00439          OR  (TEST-FIELD-07 = "RECORD ") *> sic!
00440          OR  (TEST-FIELD-21 = "ALTERNATE RECORD KEY ")
00441          OR  (TEST-FIELD-17 = "ALTERNATE RECORD ") *> sic!
00442              MOVE 1                      TO  ICOUNT
00443              STRING TEST-FIELD DELIMITED BY TWO-SPACES
00444                     " OF ISAM-FILE" DELIMITED BY SIZE
00445                                INTO  SELECT-ARRAY2-LINE (S2X)
00446                                WITH  POINTER ICOUNT
00447              SET S2X             UP  BY 1
00448              ADD 12              TO  ICOUNT
00449              SET PTR1, S1Y       UP  BY ICOUNT
00450          END-IF
00451
00452          IF  (TEST-FIELD-16 = "WITH DUPLICATES ")
00453          OR  (TEST-FIELD-11 = "DUPLICATES ")
00454          OR  (TEST-FIELD-14 = "SUPPRESS WHEN ")
00455              STRING TEST-FIELD DELIMITED BY TWO-SPACES
00456                                INTO  SELECT-ARRAY2-LINE (S2X)
00457                                WITH  POINTER ICOUNT
00458              SET S2X             UP  BY 1
00459              SET PTR1, S1Y       UP  BY ICOUNT
00460          END-IF
00461
00462          SET PTR1, S1Y       UP  BY 1
00463      END-PERFORM.
00464
00465      PERFORM VARYING S2Y FROM SELECT-ARRAY-CHARACTERS BY -1
00466              UNTIL SELECT-ARRAY2-CHAR (S2Y) NOT = SPACE
00467                 OR S2Y = 1
00468          CONTINUE
00469      END-PERFORM.
00470
00471      SET S2Y UP BY 1. MOVE "." TO SELECT-ARRAY2-CHAR (S2Y).
00472
00473 **************************************************************************
00474 * Process FD parameter and copylib file
00475 **************************************************************************
00476  120-FD-PROCESS SECTION.
00477  120-ENTRY.
00478      MOVE SPACES                 TO  FD-ARRAY.
00479
00480      IF  FD-PATH-NAME = "NOFD" OR "nofd"
00481          GO TO 120-EXIT
00482      END-IF.
00483
00484      MOVE  FD-PATH-NAME          TO  COPYLIB-MEMBER
00485
00486      PERFORM 140-OPEN-COPYLIB-FILE.
00487
00488      PERFORM WITH TEST AFTER VARYING FDX FROM 1 BY 1
00489                UNTIL FDX > FD-ARRAY-LINES
00490                   OR COPYLIB-RECORD = HIGH-VALUES
00491          PERFORM 220-READ-COPYLIB-RECORD
00492          IF  COPYLIB-RECORD NOT = HIGH-VALUES
00493              MOVE COPYLIB-RECORD     TO  FD-LINE (FDX)
00494          END-IF
00495      END-PERFORM.
00496
00497      CLOSE COPYLIB-FILE
00498
00499      IF  COPYLIB-RECORD NOT = HIGH-VALUES
00500          DISPLAY " "
00501          DISPLAY "Error: More than " FD-ARRAY-LINES
00502                  " lines in FD copylib statement!"
00503          MOVE 08 TO RETURN-CODE
00504          DISPLAY " "
00505          STOP RUN
00506      END-IF.
00507
00508      MOVE FUNCTION UPPER-CASE (FD-ARRAY) TO FD-ARRAY.
00509      INSPECT FD-ARRAY REPLACING ALL "." BY " "
00510      INSPECT FD-ARRAY REPLACING ALL "," BY " "
00511
00512 *    Erase the "FD FILENAME" from the FD-ARRAY because we will
00513 *    generate "FD  SEQ-FILE" and "FD  ISAM-FILE", which filenames
00514 *    will probably not match what is in the copylib record
00515
00516      MOVE "Y"                    TO  SUPPRESS-FD-REC.
00517      SET PTR1                    TO  ADDRESS OF FD-ARRAY
00518      SET FDY                     TO  1
00519      PERFORM UNTIL (FDY > FD-ARRAY-CHARACTERS)
00520                 OR (SUPPRESS-FD-REC = SPACE)
00521          SET ADDRESS OF TEST-FIELD TO PTR1
00522          IF  TEST-FIELD-03 = "FD "
00523              MOVE SPACES         TO  SUPPRESS-FD-REC
00524              PERFORM 3 TIMES
00525                  MOVE SPACES TO FD-CHAR (FDY)
00526                  SET  PTR1, FDY UP BY 1
00527              END-PERFORM
00528              PERFORM UNTIL FD-CHAR (FDY) NOT = SPACES
00529                         OR FDY > FD-ARRAY-CHARACTERS
00530                  SET  PTR1, FDY UP BY 1
00531              END-PERFORM
00532              PERFORM UNTIL FD-CHAR (FDY) = SPACES
00533                         OR FDY > FD-ARRAY-CHARACTERS
00534                  MOVE SPACES TO FD-CHAR (FDY)
00535                  SET  PTR1, FDY UP BY 1
00536              END-PERFORM
00537          END-IF
00538          SET  PTR1, FDY UP BY 1
00539      END-PERFORM.
00540
00541  120-EXIT.
00542      EXIT.
00543
00544 **************************************************************************
00545 * Process RECORD parameter and copylib file
00546 **************************************************************************
00547  130-RECORD-PROCESS SECTION.
00548  130-ENTRY.
00549      MOVE RECORD-PATH-NAME       TO  COPYLIB-MEMBER
00550
00551      PERFORM 140-OPEN-COPYLIB-FILE
00552
00553      MOVE COPYLIB-PATH-NAME    TO  RECORD-PATH-NAME
00554
00555      CLOSE COPYLIB-FILE.
00556
00557  140-OPEN-COPYLIB-FILE SECTION.
00558  140-ENTRY.
00559      MOVE SPACE                  TO  FIXED-FORMAT.
00560      MOVE COPYLIB-MEMBER         TO  COPYLIB-PATH-NAME.
00561
00562      PERFORM 150-COPYLIB-EXTENSIONS.
00563
00564      IF  COPYLIB-FILE-STATUS = ZERO GO TO 140-EXIT.
00565
00566      MOVE SPACES TO COPYLIB-PATH-NAME.
00567
00568      STRING COBCPY-FOLDER "\" COPYLIB-MEMBER
00569             DELIMITED BY SPACE  INTO  COPYLIB-PATH-NAME.
00570
00571      PERFORM 150-COPYLIB-EXTENSIONS.
00572
00573      IF  COPYLIB-FILE-STATUS = ZERO GO TO 140-EXIT.
00574
00575      MOVE SPACES TO COPYLIB-PATH-NAME.
00576
00577      STRING COB-COPY-DIR-FOLDER "\" COPYLIB-MEMBER
00578             DELIMITED BY SPACE  INTO  COPYLIB-PATH-NAME.
00579
00580      PERFORM 150-COPYLIB-EXTENSIONS.
00581
00582      IF  COPYLIB-FILE-STATUS > ZERO
00583          DISPLAY " "
00584          DISPLAY
00585          "Error: copylib file not found! " COPYLIB-MEMBER
00586          DISPLAY "FILE-STATUS-CODE=" COPYLIB-FILE-STATUS
00587          MOVE 08 TO RETURN-CODE
00588          DISPLAY " "
00589          STOP RUN
00590      END-IF.
00591
00592  140-EXIT.
00593      EXIT.
00594
00595  150-COPYLIB-EXTENSIONS SECTION.
00596  150-ENTRY.
00597      IF  OS = "LINUX" INSPECT COPYLIB-PATH-NAME
00598                       REPLACING ALL "\" BY "/".
00599
00600      OPEN INPUT COPYLIB-FILE. *>Start with unextended filename
00601
00602      IF  COPYLIB-FILE-STATUS = ZERO GO TO 150-EXIT.
00603
00604      INSPECT COPYLIB-PATH-NAME REPLACING FIRST "     " BY ".COB ".
00605
00606      OPEN INPUT COPYLIB-FILE.
00607
00608      IF  COPYLIB-FILE-STATUS = ZERO GO TO 150-EXIT.
00609
00610      INSPECT COPYLIB-PATH-NAME REPLACING FIRST ".COB " BY ".CBL ".
00611
00612      OPEN INPUT COPYLIB-FILE.
00613
00614      IF  COPYLIB-FILE-STATUS = ZERO GO TO 150-EXIT.
00615
00616      INSPECT COPYLIB-PATH-NAME REPLACING ALL ".CBL " BY ".CPY ".
00617
00618      IF  COPYLIB-FILE-STATUS = ZERO GO TO 150-EXIT.
00619
00620      IF  OS NOT = "LINUX" GO TO 150-EXIT.
00621
00622      INSPECT COPYLIB-PATH-NAME REPLACING FIRST ".CPY " BY ".cob ".
00623
00624      OPEN INPUT COPYLIB-FILE.
00625
00626      IF  COPYLIB-FILE-STATUS = ZERO GO TO 150-EXIT.
00627
00628      INSPECT COPYLIB-PATH-NAME REPLACING FIRST ".cob " BY ".cbl ".
00629
00630      OPEN INPUT COPYLIB-FILE.
00631
00632      IF  COPYLIB-FILE-STATUS = ZERO GO TO 150-EXIT.
00633
00634      INSPECT COPYLIB-PATH-NAME REPLACING ALL ".cbl " BY ".cpy ".
00635
00636 *    When we exit, the file is open if copylib-file-status = zero
00637  150-EXIT.
00638      EXIT.
00639
00640  200-GENERATE-COBOL-PROGRAM SECTION.
00641  200-ENTRY.
00642      SET ENVIRONMENT  "SEQ_PATH_NAME" TO  SEQ-PATH-NAME.
00643      SET ENVIRONMENT "ISAM_PATH_NAME" TO ISAM-PATH-NAME.
00644
00645      IF  OUTPUT-PATH-NAME NOT = SPACES
00646          MOVE OUTPUT-PATH-NAME   TO  COBOL-PATH-NAME
00647      END-IF.
00648
00649      OPEN OUTPUT COBOL-FILE.
00650
00651      MOVE SPACES                 TO  COBOL-RECORD.
00652
00653      MOVE "IDENTIFICATION DIVISION." TO COBOL-TEXT.
00654      WRITE COBOL-RECORD
00655      MOVE "PROGRAM-ID.    GNUDBLDX." TO COBOL-TEXT.
00656      WRITE COBOL-RECORD
00657      MOVE "AUTHOR.       CARL D. GOLDIN." TO COBOL-TEXT.
00658      WRITE COBOL-RECORD
00659      MOVE "*>DATE-WRITTEN.  Jan 05, 2020." TO COBOL-TEXT.
00660      WRITE COBOL-RECORD
00661      MOVE "*>LAST-REVISION. Jan 07, 2020." TO COBOL-TEXT.
00662      WRITE COBOL-RECORD
00663      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00664      MOVE
00665      "*> Copyright (c) Carl D. Goldin, 2020. " TO COBOL-TEXT.
00666      WRITE COBOL-RECORD
00667      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00668
00669      MOVE "ENVIRONMENT DIVISION." TO COBOL-TEXT.
00670      WRITE COBOL-RECORD
00671      MOVE "INPUT-OUTPUT SECTION." TO COBOL-TEXT.
00672      WRITE COBOL-RECORD
00673      MOVE "FILE-CONTROL." TO COBOL-TEXT.
00674      WRITE COBOL-RECORD
00675
00676      MOVE "    SELECT SEQ-FILE ASSIGN TO SEQ-PATH-NAME"
00677           TO COBOL-TEXT. WRITE COBOL-RECORD
00678      MOVE "           ORGANIZATION IS LINE SEQUENTIAL."
00679           TO COBOL-TEXT. WRITE COBOL-RECORD
00680      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00681
00682      MOVE "    SELECT ISAM-FILE ASSIGN TO ISAM-PATH-NAME"
00683           TO COBOL-TEXT.  WRITE COBOL-RECORD
00684      MOVE "           ORGANIZATION IS INDEXED" TO COBOL-TEXT.
00685      WRITE COBOL-RECORD
00686
00687      MOVE "           ACCESS MODE IS DYNAMIC" TO COBOL-TEXT.
00688      WRITE COBOL-RECORD
00689
00690      PERFORM VARYING S2X FROM 1 BY 1
00691                UNTIL S2X > SELECT-ARRAY-LINES
00692          IF  SELECT-ARRAY2-LINE (S2X) NOT = SPACES
00693              MOVE SPACES         TO  COBOL-TEXT
00694              STRING "           " SELECT-ARRAY2-LINE (S2X)
00695                                INTO  COBOL-TEXT
00696              WRITE COBOL-RECORD
00697          END-IF
00698      END-PERFORM.
00699
00700      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00701      MOVE "DATA DIVISION." TO COBOL-TEXT. WRITE COBOL-RECORD
00702      MOVE "FILE SECTION."  TO COBOL-TEXT.  WRITE COBOL-RECORD
00703
00704      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00705      MOVE "FD  SEQ-FILE" TO COBOL-TEXT. WRITE COBOL-RECORD
00706
00707      PERFORM VARYING FDX FROM 1 BY 1
00708                UNTIL FDX > FD-ARRAY-LINES
00709          IF  FD-LINE (FDX) NOT = SPACES
00710              MOVE FD-LINE (FDX) TO FREE-FORMAT-TEXT
00711              WRITE COBOL-RECORD
00712          END-IF
00713      END-PERFORM.
00714
00715      MOVE SPACES                 TO  COBOL-RECORD
00716      MOVE "    ."                TO  COBOL-TEXT.
00717      WRITE COBOL-RECORD
00718
00719      MOVE "01  SEQ-RECORD." TO COBOL-TEXT. WRITE COBOL-RECORD
00720
00721      PERFORM 210-WRITE-COPYLIB-RECORDS.
00722
00723      MOVE "FD  ISAM-FILE"  TO COBOL-TEXT. WRITE COBOL-RECORD
00724
00725      PERFORM VARYING FDX FROM 1 BY 1
00726                UNTIL FDX > FD-ARRAY-LINES
00727          IF  FD-LINE (FDX) NOT = SPACES
00728              MOVE FD-LINE (FDX) TO FREE-FORMAT-TEXT
00729              WRITE COBOL-RECORD
00730          END-IF
00731      END-PERFORM.
00732
00733      MOVE SPACES                 TO  COBOL-RECORD
00734      MOVE "    ."                TO  COBOL-TEXT.
00735      WRITE COBOL-RECORD
00736
00737      MOVE "01  ISAM-RECORD." TO COBOL-TEXT. WRITE COBOL-RECORD
00738
00739      PERFORM 210-WRITE-COPYLIB-RECORDS.
00740
00741      MOVE "WORKING-STORAGE SECTION." TO COBOL-TEXT.
00742      WRITE COBOL-RECORD
00743
00744      MOVE "01  REPLY          PIC X." TO COBOL-TEXT.
00745      WRITE COBOL-RECORD
00746      STRING "01   SEQ-PATH-NAME PIC X(80) value "
00747              """SEQ_PATH_NAME""."
00748                                INTO  COBOL-TEXT
00749      WRITE COBOL-RECORD
00750      MOVE SPACES                 TO  COBOL-TEXT
00751      STRING "01  ISAM-PATH-NAME PIC X(80) value "
00752              """ISAM_PATH_NAME""."
00753                                INTO  COBOL-TEXT
00754      WRITE COBOL-RECORD
00755      MOVE "01  RECORDS-ADDED  PIC 9(7)  VALUE ZERO."
00756            TO COBOL-TEXT. WRITE COBOL-RECORD
00757      MOVE "01  RECORDS-READ   PIC 9(7)  VALUE ZERO."
00758            TO COBOL-TEXT. WRITE COBOL-RECORD
00759      MOVE "01  SEQ-FILE-EOF   PIC X     VALUE ZERO."
00760             TO COBOL-TEXT. WRITE COBOL-RECORD
00761      MOVE SPACES TO COBOL-TEXT.
00762      STRING "01  LOAD-OR-DUMP   PIC X(08) VALUE """
00763                 LOAD-OR-DUMP """." INTO COBOL-TEXT
00764      WRITE COBOL-RECORD
00765      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00766
00767      MOVE "PROCEDURE DIVISION." TO COBOL-TEXT.
00768      WRITE COBOL-RECORD
00769      MOVE "000-MAINLINE SECTION." TO COBOL-TEXT.
00770      WRITE COBOL-RECORD
00771      MOVE "000-ENTRY." TO COBOL-TEXT.
00772      WRITE COBOL-RECORD
00773      MOVE "    MOVE ZERO TO  RETURN-CODE." TO COBOL-TEXT.
00774      WRITE COBOL-RECORD
00775
00776      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00777      STRING "    ACCEPT  SEQ-PATH-NAME FROM ENVIRONMENT "
00778             """SEQ_PATH_NAME""" INTO COBOL-TEXT.
00779      WRITE COBOL-RECORD
00780      MOVE   "        ON EXCEPTION DISPLAY "" "" " TO COBOL-TEXT.
00781      WRITE COBOL-RECORD
00782      STRING "        DISPLAY ""Environment variable "
00783             "SEQ_PATH_NAME missing!""" INTO COBOL-TEXT.
00784      WRITE COBOL-RECORD
00785      MOVE   "        DISPLAY "" "" " TO COBOL-TEXT.
00786      WRITE COBOL-RECORD
00787      MOVE   "        MOVE 8 TO RETURN-CODE" TO COBOL-TEXT.
00788      WRITE COBOL-RECORD
00789      MOVE   "        STOP RUN." TO COBOL-TEXT. WRITE COBOL-RECORD
00790
00791      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00792      STRING "    ACCEPT ISAM-PATH-NAME FROM ENVIRONMENT "
00793             """ISAM_PATH_NAME""" INTO COBOL-TEXT.
00794      WRITE COBOL-RECORD
00795      MOVE   "        ON EXCEPTION DISPLAY "" """  TO COBOL-TEXT.
00796      WRITE COBOL-RECORD
00797      STRING "        DISPLAY ""Environment variable "
00798             "ISAM_PATH_NAME missing!""" INTO COBOL-TEXT.
00799      WRITE COBOL-RECORD
00800      MOVE   "        DISPLAY "" """ TO COBOL-TEXT.
00801      WRITE COBOL-RECORD
00802      MOVE   "        MOVE 8 TO RETURN-CODE" TO COBOL-TEXT.
00803      WRITE COBOL-RECORD
00804      MOVE   "        STOP RUN." TO COBOL-TEXT. WRITE COBOL-RECORD
00805
00806      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00807      MOVE "    IF  LOAD-OR-DUMP = ""DUMP""" TO COBOL-TEXT.
00808      WRITE COBOL-RECORD
00809      MOVE "        PERFORM 100-DUMP-ROUTINE" TO COBOL-TEXT.
00810      WRITE COBOL-RECORD
00811      MOVE "        PERFORM 200-PRESS-ANY-KEY" TO COBOL-TEXT.
00812      WRITE COBOL-RECORD
00813      MOVE "        STOP RUN" TO COBOL-TEXT. WRITE COBOL-RECORD
00814      MOVE "    END-IF." TO COBOL-TEXT. WRITE COBOL-RECORD
00815      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD.
00816
00817      MOVE "*> LOAD and APPEND procedure" TO COBOL-TEXT.
00818      WRITE COBOL-RECORD
00819
00820      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD.
00821
00822      MOVE "    OPEN INPUT SEQ-FILE." TO COBOL-TEXT.
00823      WRITE COBOL-RECORD
00824      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00825
00826      MOVE "    IF  LOAD-OR-DUMP = ""APPEND""" TO COBOL-TEXT.
00827      WRITE COBOL-RECORD
00828      MOVE "        OPEN EXTEND ISAM-FILE" TO COBOL-TEXT.
00829      WRITE COBOL-RECORD
00830      MOVE "    ELSE"    TO COBOL-TEXT.
00831      WRITE COBOL-RECORD
00832      MOVE "        OPEN OUTPUT ISAM-FILE" TO COBOL-TEXT.
00833      WRITE COBOL-RECORD
00834      MOVE "    END-IF." TO COBOL-TEXT.
00835      WRITE COBOL-RECORD
00836      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD.
00837
00838      MOVE "000-LOOP." TO COBOL-TEXT. WRITE COBOL-RECORD
00839      MOVE "    READ SEQ-FILE RECORD" TO COBOL-TEXT.
00840      WRITE COBOL-RECORD
00841      MOVE "         AT END GO TO 000-EOF" TO COBOL-TEXT.
00842      WRITE COBOL-RECORD
00843      MOVE "    END-READ." TO COBOL-TEXT.
00844      WRITE COBOL-RECORD
00845      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00846
00847      MOVE "    IF  SEQ-RECORD = SPACES OR LOW-VALUES"
00848           TO COBOL-TEXT. WRITE COBOL-RECORD
00849      MOVE "        GO TO 000-LOOP." TO COBOL-TEXT.
00850      WRITE COBOL-RECORD
00851      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD.
00852
00853      MOVE "    WRITE ISAM-RECORD FROM SEQ-RECORD." TO COBOL-TEXT.
00854      WRITE COBOL-RECORD
00855      MOVE "    ADD 1 TO  RECORDS-ADDED." TO COBOL-TEXT.
00856      WRITE COBOL-RECORD
00857      MOVE "    GO TO 000-LOOP." TO COBOL-TEXT.
00858      WRITE COBOL-RECORD
00859      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00860
00861      MOVE "000-EOF." TO COBOL-TEXT.  WRITE COBOL-RECORD
00862      STRING "    DISPLAY RECORDS-ADDED "" records loaded to """
00863             "ISAM-PATH-NAME."
00864             DELIMITED BY SIZE INTO COBOL-TEXT
00865      WRITE COBOL-RECORD.
00866      MOVE "    DISPLAY "" ""." TO COBOL-TEXT.
00867      WRITE COBOL-RECORD
00868
00869      MOVE "    CLOSE SEQ-FILE, ISAM-FILE." TO COBOL-TEXT.
00870      WRITE COBOL-RECORD
00871      MOVE "    PERFORM 200-PRESS-ANY-KEY" TO COBOL-TEXT.
00872      WRITE COBOL-RECORD
00873      MOVE "    STOP RUN." TO COBOL-TEXT.
00874      WRITE COBOL-RECORD
00875      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD.
00876
00877      MOVE "100-DUMP-ROUTINE SECTION." TO COBOL-TEXT.
00878      WRITE COBOL-RECORD
00879      MOVE "100-ENTRY."  TO COBOL-TEXT. WRITE COBOL-RECORD
00880      MOVE "    OPEN OUTPUT SEQ-FILE."  TO COBOL-TEXT.
00881      WRITE COBOL-RECORD
00882      MOVE "    MOVE ZERO TO RECORDS-READ." TO COBOL-TEXT.
00883      WRITE COBOL-RECORD
00884      MOVE "    OPEN INPUT ISAM-FILE." TO COBOL-TEXT.
00885      WRITE COBOL-RECORD
00886      MOVE "100-LOOP." TO COBOL-TEXT. WRITE COBOL-RECORD
00887      MOVE "    MOVE SPACES TO SEQ-RECORD, ISAM-RECORD."
00888           TO COBOL-TEXT. WRITE COBOL-RECORD
00889
00890      MOVE "    READ ISAM-FILE" TO COBOL-TEXT.
00891      WRITE COBOL-RECORD
00892      MOVE "         AT END GO TO 100-EOF" TO COBOL-TEXT.
00893      WRITE COBOL-RECORD
00894      MOVE "    END-READ" TO COBOL-TEXT. WRITE COBOL-RECORD
00895      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD.
00896      MOVE "    ADD  1  TO  RECORDS-READ." TO COBOL-TEXT.
00897      WRITE COBOL-RECORD
00898      MOVE "    WRITE SEQ-RECORD FROM ISAM-RECORD" TO COBOL-TEXT.
00899      WRITE COBOL-RECORD
00900      MOVE "    GO TO 100-LOOP." TO COBOL-TEXT.
00901      WRITE COBOL-RECORD
00902      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD.
00903      MOVE "100-EOF." TO COBOL-TEXT. WRITE COBOL-RECORD
00904      STRING "    DISPLAY RECORDS-READ "" records written to """
00905             "SEQ-PATH-NAME" DELIMITED BY SIZE INTO COBOL-TEXT
00906      WRITE COBOL-RECORD.
00907      MOVE "    DISPLAY "" ""." TO COBOL-TEXT. WRITE COBOL-RECORD
00908      MOVE "    CLOSE SEQ-FILE, ISAM-FILE."  TO COBOL-TEXT.
00909      WRITE COBOL-RECORD
00910      MOVE "    PERFORM 200-PRESS-ANY-KEY" TO COBOL-TEXT.
00911      WRITE COBOL-RECORD
00912      MOVE "    STOP RUN." TO COBOL-TEXT. WRITE COBOL-RECORD
00913      MOVE SPACES TO COBOL-TEXT. WRITE COBOL-RECORD
00914
00915      MOVE "200-PRESS-ANY-KEY SECTION." TO COBOL-TEXT.
00916      WRITE COBOL-RECORD
00917      MOVE "200-ENTRY." TO COBOL-TEXT. WRITE COBOL-RECORD
00918      MOVE "    DISPLAY 'Press any key to continue'." TO COBOL-TEXT.
00919      WRITE COBOL-RECORD
00920      MOVE "    ACCEPT REPLY." TO COBOL-TEXT.
00921      WRITE COBOL-RECORD
00922
00923      CLOSE COBOL-FILE.
00924
00925      IF  OUTPUT-PATH-NAME = SPACES
00926          IF  CREATE-FREE-FORMAT-FILE = "Y"
00927              CALL "SYSTEM" USING
00928                   "COBC -xjF GNUDBLDX.COB -T GNUDBLDX.LST"
00929          ELSE
00930              CALL "SYSTEM" USING
00931                   "COBC -xj  GNUDBLDX.COB -T GNUDBLDX.LST"
00932          END-IF
00933      ELSE
00934          DISPLAY "Cobol load/dump program file created: "
00935                   OUTPUT-PATH-NAME
00936      END-IF.
00937
00938  210-WRITE-COPYLIB-RECORDS SECTION.
00939  210-ENTRY.
00940      MOVE  RECORD-PATH-NAME      TO  COPYLIB-PATH-NAME.
00941
00942      OPEN INPUT COPYLIB-FILE.
00943
00944      MOVE SPACE                  TO  FIXED-FORMAT.
00945
00946      IF  COPYLIB-FILE-STATUS > ZERO
00947          DISPLAY " "
00948          DISPLAY
00949          "Error: copylib file not found! " COPYLIB-PATH-NAME
00950          DISPLAY "FILE-STATUS-CODE=" COPYLIB-FILE-STATUS
00951          MOVE 08 TO RETURN-CODE
00952          DISPLAY " "
00953          STOP RUN
00954      END-IF.
00955
00956      IF  COLUMNS-73-PLUS NOT = SPACES
00957          MOVE "Y"                TO  CREATE-FREE-FORMAT-FILE
00958      END-IF.
00959
00960      MOVE "Y"                    TO  SUPPRESS-01-REC.
00961
00962      PERFORM WITH TEST AFTER UNTIL COPYLIB-RECORD = HIGH-VALUES
00963          PERFORM 220-READ-COPYLIB-RECORD
00964          IF  NOT (COPYLIB-RECORD = HIGH-VALUES OR LOW-VALUES)
00965              MOVE COPYLIB-RECORD TO FREE-FORMAT-TEXT
00966              WRITE COBOL-RECORD
00967          END-IF
00968      END-PERFORM.
00969
00970      CLOSE COPYLIB-FILE
00971
00972      MOVE SPACES TO COBOL-RECORD. WRITE COBOL-RECORD.
00973
00974  220-READ-COPYLIB-RECORD SECTION.
00975  220-ENTRY.
00976      READ COPYLIB-FILE
00977           AT END MOVE HIGH-VALUES TO COPYLIB-RECORD
00978                  GO TO 220-EXIT
00979      END-READ.
00980
00981      IF  SEQUENCE-NUMBER IS NUMERIC
00982          MOVE "Y"                TO  FIXED-FORMAT
00983          *> THIS COULD BE A PROBLEM IF THE FIRST RECORD DOESN'T
00984          *> HAVE A SEQUENCE NUMBER, OR IF THE FILE IS FIXED-
00985          *> FORMAT WITHOUT SEQUENCE NUMBERS (AND COLUMNS 73-80
00986          *> ARE USED FOR PROGRAM-ID OR COMMENTS)!
00987      END-IF
00988
00989 *    Convert fixed-format record to free-format
00990
00991      IF  FIXED-FORMAT = "Y"
00992 *        this handles the case where the first record has a sequence
00993 *        number, but subsequent records do not.
00994          MOVE SPACES             TO  COLUMN-6
00995          IF  COLUMN-7 = "*"
00996              MOVE "*"            TO  COLUMN-6
00997              MOVE ">"            TO  COLUMN-7
00998              MOVE COMMENT-TEXT   TO  COPYLIB-RECORD
00999          ELSE
01000              MOVE COPYLIB-TEXT   TO  COPYLIB-RECORD
01001          END-IF
01002      END-IF.
01003
01004 *    Remove the "01  record-name" from the first copylib member
01005
01006      IF  (SUPPRESS-01-REC = "Y")
01007          MOVE ZERO               TO  ICOUNT
01008          INSPECT COPYLIB-RECORD TALLYING ICOUNT FOR ALL " 01 "
01009          IF (ICOUNT > ZERO) OR (COLUMN-1-3 = "01 ")
01010              MOVE LOW-VALUES     TO  COPYLIB-RECORD
01011              MOVE SPACES         TO  SUPPRESS-01-REC
01012          END-IF
01013      END-IF.
01014
01015  220-EXIT.
01016      EXIT.
01017
01018 ******************* End of Program GNUDBLD ***********************
